from setuptools import setup, find_packages

setup(
    name="django-class-registry",
    version='1.1.2',
    author="IMT Computer Services",
    author_email="imtapps@apps-system.com",
    description="Register any classes like Django's admin.",
    long_description=open('README.txt', 'r').read(),
    url="http://github.com/imtapps/django-class-registry",
    packages=find_packages(),
    install_requires=[],
    tests_requires=[],
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],
)
